#!/usr/bin/env python
#----------------------------------------------------------------------------
# Name:         OritatamiSim.py
# Purpose:      A basic program for drawing shapes, scaling and rotating them,
#               and concatenating sets of bead sequences to create an Oritatami
#               system which self-assembles the shape (at constant scale factor
#               and with a constant set of bead types, but with a sequence which
#               grows linearly with the size of the shape).
#
# Author:       Matt Patitz
#
# Created:      
# RCS-ID:       
# Copyright:    
# License:      
#----------------------------------------------------------------------------

"""
This program can load the definition of an oritatami system from a file and
simulate the growth of an assembly based on it.

Requirements:
    This application requires Python 2.x

Dependencies:
    wxPython    (download installer for your platform here: https://wxpython.org/Phoenix/snapshot-builds/)
    pil         (via pillow, install using 'pip install pillow', pip can be found in your Python27\Scripts dir)
    svgwrite    (via 'pip install svgwrite')
"""

"""
MJP TODO list:
    - utilize thread-safe UI calls
        - reowork DrawBeadCycle
    - fix parent-child links and bonds to either never or always cover beads
    - don't make path to system definition an absolute path
        - when creating a new system, make the assembly's path to the system definition relative?
    - CRASH: when bead cycle window is open during fast forward
    - when parameters DelayLength, Arity, LockWithoutBinding are set, should they be remembered when creating new?
        - test setting and saving
    - add saving of systems
        - implement flag to denote when a system has been edited to warn when (re)loading
        - when loading, don't allow whitespace in names
    - add functionality to window for typing and viewing primary structure
        - add help button
        - BUG: sometimes after reset (or reload?) the entire sequence becomes drawn in bold
        - disallow any whitespace entry
    - fix deletion of nodes when in drawing mode:
        - if a bead in the middle of a string is deleted, going forward is messed up
        - redrawing is very inefficient
        - deleting from the middle will cause additional problems when trying to insert back in...
    - terminate thread on exit if it's running!
        - doesn't get killed on breakpoint or when can't find free loc
    - on mouse over:  display number of addition (requires cache)
    - make help window a single instance which you can turn on and off
    - add option for toggling locking by only secondary structure of floppy segment
    - u: undo, r: reverse (will need to cache additions, allow seed change midway)
    - set icon in py2exe script so it will show up in task bar
"""

import wx
import wx.richtext as rt
import os
import random
import math
import re
import ast # TODO: This was needed for debugging remove when done using 
import sys
import thread
from collections import deque
import itertools
import wx.aui
import time
import threading
import wx.lib.scrolledpanel as scrolled
from wx.lib.agw.shapedbutton import SButton, SBitmapButton
import wx.lib.agw.cubecolourdialog as CCD
import wx.html
from OritatamiSimNodes import *
from OritatamiSimUtils import *
from OritatamiSimGrid import *
from OritatamiSimCanvas import *
from SimulatorFramework import SimApp


# IDs
ORIT_ID_FR             = 100
ORIT_ID_REV            = 101
ORIT_ID_STOP           = 102
ORIT_ID_FOR            = 103
ORIT_ID_FF             = 104
ORIT_ID_NEW            = 105
ORIT_ID_LOAD           = 106
ORIT_ID_SAVE           = 107
ORIT_ID_ZOOMIN         = 108
ORIT_ID_ZOOMOUT        = 109
ORIT_ID_MODE_PAN_ZOOM  = 110
ORIT_ID_MODE_DRAW_BEAD = 111
ORIT_ID_MODE_DRAW_LINK = 112
ORIT_ID_MODE_DRAW_BOND = 113
ORIT_ID_RESET          = 114
ORIT_ID_RESET_BEADSEQ  = 115
ORIT_ID_RELOAD_BEADSEQ = 116
ORIT_ID_CANCEL_BEADSEQ = 117


#---------------------------------------------------------------------------

class OritatamiBeadBox(wx.StaticBox):
    
    def __init__(self, beadType, panel, panelSizer, parentApp, beadBoxList):
        wx.StaticBox.__init__(self, panel, wx.ID_ANY, "", size=(200,100))
        self.parentPanel = panel
        self.parentPanelSizer = panelSizer
        self.parentApp = parentApp
        self.beadBoxList = beadBoxList
        self.beadType = beadType
        buttonLabel = "Add"
        if beadType is not None:
            buttonLabel = "Set"
        else:
            self.beadType = ONodeType("", "Black", [])
        #self.beadBox = wx.StaticBox(panel, wx.ID_ANY, "", size=(100,100))        
        self.beadBoxSizer = wx.StaticBoxSizer(self, wx.VERTICAL)

        self.beadBoxSizer.Add((0, 10), 0)  
        nameSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.applyButton = wx.Button(self, wx.ID_ANY, buttonLabel, size=(35,-1))
        self.applyButton.Bind(wx.EVT_MOUSE_EVENTS, self.OnApply)
        nameSizer.Add(self.applyButton, 1)

        self.colorButton = SBitmapButton(self, wx.ID_ANY, self.GetBeadButtonBitmap(self.beadType.color))
        self.colorButton.Bind(wx.EVT_MOUSE_EVENTS, self.PickColor)  
        nameSizer.Add(self.colorButton, 1)
        nameSizer.AddSpacer(5)

        lblname = wx.StaticText(self, label="Name:")
        lblname.Bind(wx.EVT_MOUSE_EVENTS, self.parentApp.OnBeadPanelClickStatText)
        nameSizer.Add(lblname, 0, wx.ALIGN_CENTER_VERTICAL, 5)
        nameSizer.AddSpacer(5)

        self.editName = wx.TextCtrl(self, size=(60, -1), value=self.beadType.name)
        nameSizer.Add(self.editName)
        
        if beadType is not None:
            self.deleteButton = wx.Button(self, wx.ID_ANY, "X", size=(2,30))
            self.deleteButton.Bind(wx.EVT_MOUSE_EVENTS, SimUtils.globalApp.OnBeadTypeDelete)
            nameSizer.Add(self.deleteButton, 1)

        self.beadBoxSizer.Add(nameSizer)
        self.beadBoxSizer.AddSpacer(5)
        
        bondsSizer = wx.BoxSizer(wx.HORIZONTAL)
        lblbonds = wx.StaticText(self, label="Bonds:")
        lblbonds.Bind(wx.EVT_MOUSE_EVENTS, self.parentApp.OnBeadPanelClickStatText)
        bondsSizer.Add(lblbonds, 0, wx.ALIGN_CENTER_VERTICAL, 5)
        bondsSizer.AddSpacer(5)

        bondTypeList = ""
        for bt in self.beadType.bondToTypes:
            if len(bondTypeList) > 0:
                bondTypeList = bondTypeList + ","
            bondTypeList = bondTypeList + bt
        self.editBonds = wx.TextCtrl(self, size=(155, -1), value=bondTypeList)
        bondsSizer.Add(self.editBonds)
        self.beadBoxSizer.Add(bondsSizer)

        self.beadBoxSizer.Add((0, 10), 0)
        panelSizer.Add(self.beadBoxSizer, 2)

    def OnApply(self, event):
        if event.LeftDown():
            box = event.EventObject.Parent
            bAdding = False
            if box.beadType.name == "":  # Creating a new bead type
                bAdding = True
            name = str(box.editName.GetValue())
            if len(name) == 0:
                SimUtils.globalApp.ShowErrorDialog("Bead type name cannot be blank")
                return
            elif (" " in name) or ("," in name) or (";" in name) :
                SimUtils.globalApp.ShowErrorDialog("Bead type name cannot contain a space, comma, or semicolon")
                return
            bondToTypes = str(box.editBonds.GetValue())
            if (" " in bondToTypes) or (";" in bondToTypes):
                SimUtils.globalApp.ShowErrorDialog("Bond types list should be a comma\nseparated list of bead type names")
                return
            typeList = []
            if len(bondToTypes) > 0:
                typeList = [str(t) for t in bondToTypes.split(",")]
            notFoundList = []
            for t in typeList:
                bFound = False
                for bb in self.beadBoxList:
                    if t == self.beadBoxList[bb].beadType.name:
                        bFound = True
                if bFound == False:
                    notFoundList.append(t)
            if len(notFoundList) > 0:
                SimUtils.globalApp.ShowWarningDialog("Warning: bond type list contains names of nonexistent bead types:\n" + str(notFoundList))

            if bAdding:
                
                for bb in self.beadBoxList:
                    if self.beadBoxList[bb].beadType.name == name:
                        SimUtils.globalApp.ShowErrorDialog("A bead type named '" + name + "' already exists.\nYou must choose a new name for a new type.")
                        return

                newBeadType = ONodeType(name, box.beadType.color, typeList)
                bb = OritatamiBeadBox(newBeadType, box.parentPanel, box.parentPanelSizer, box.parentApp, box.beadBoxList)
                box.beadBoxList[name] = bb
                box.parentPanelSizer.Layout()
                box.parentApp.frame._mgr.Update()
                
                # Reset the form
                box.beadType = ONodeType("", "Black", [])
                box.editName.SetValue("")
                box.editBonds.SetValue("")
                box.colorButton.SetBitmapLabel(self.GetBeadButtonBitmap(box.beadType.color))

                # Put the new bead type into simulator's list too
                box.parentApp.canvas.assembly.ONodeTypes[name] = newBeadType
            else:
                self.UpdateBeadType(box, box.beadType.name, name, box.beadType.color, typeList)
            
            if box.parentApp.canvas.assembly.FixBeadBondSymmetry(True, name, typeList) == True:
                for btName in box.parentApp.canvas.assembly.ONodeTypes:
                    bb = self.beadBoxList[btName]
                    btTypes = box.parentApp.canvas.assembly.ONodeTypes[btName].bondToTypes
                    btStr = ""
                    for bt in btTypes:
                        if len(btStr) > 0:
                            btStr = btStr + ","
                        btStr = btStr + bt
                    bb.editBonds.SetValue(btStr)
                    bb.editBonds.Refresh()

    
    def UpdateBeadType(self, beadBox, oldName, newName, newColor, newBondToTypes):
        #print("updating bead type " + oldName)
        #oldBeadType = beadBox.beadType

        if not oldName == newName:
            # Ask user if old bonds should be redone and/or if beads that used to bond to it should be updated to still bond to it
            #dlg = wx.MessageDialog(SimUtils.globalApp.frame, "Would you like bead types that bond to this type with its old name to bind\nto it with its new name and to update all beads in the current assembly?",
            #                                        "Update other bead types?", wx.YES_NO | wx.ICON_QUESTION)
            #result = (dlg.ShowModal() == wx.ID_YES)
            #dlg.Destroy()
            #if result == True:
            for nodeTypeName in beadBox.parentApp.canvas.assembly.ONodeTypes:
                nodeType = beadBox.parentApp.canvas.assembly.ONodeTypes[nodeTypeName]
                if nodeType.BondsToNodeType(oldName):
                    nodeType.bondToTypes.remove(oldName)
                    nodeType.bondToTypes.append(newName)
            for bb in self.beadBoxList:
                beadBox2 = self.beadBoxList[bb]
                nodeType = beadBox2.beadType
                if nodeType.BondsToNodeType(newName):
                    bondTypeList = ""
                    for bt in nodeType.bondToTypes:
                        if len(bondTypeList) > 0:
                            bondTypeList = bondTypeList + ","
                        bondTypeList = bondTypeList + bt
                    beadBox2.editBonds.SetValue(bondTypeList)
                    beadBox2.editBonds.Refresh()

            for index, item in enumerate(newBondToTypes):
                if item == oldName:
                    newBondToTypes[index] = newName

            beadBox.parentPanelSizer.Layout()
            beadBox.parentApp.frame._mgr.Update()

        newBeadType = ONodeType(newName, newColor, newBondToTypes)
        beadBox.beadType = newBeadType

        del beadBox.parentApp.canvas.assembly.ONodeTypes[oldName]
        beadBox.parentApp.canvas.assembly.ONodeTypes[newName] = newBeadType
        
        del self.beadBoxList[oldName]
        self.beadBoxList[newName] = beadBox

        for bbName in self.beadBoxList:
            bb = self.beadBoxList[bbName]
            if bb.beadType.name in newBondToTypes:
                bb.SetBackgroundColour("Yellow")
            else:
                bb.SetBackgroundColour(wx.NullColour)
            bb.Refresh()
            
        # Update the bead cycle with the new name
        for index, item in enumerate(beadBox.parentApp.canvas.assembly.beadCycle):
            if item == oldName:
                beadBox.parentApp.canvas.assembly.beadCycle[index] = newName
        beadBox.parentApp.DrawBeadCycle()
        
        # Update the assembly
        for i in range(beadBox.parentApp.canvas.assembly.minX, beadBox.parentApp.canvas.assembly.maxX+1):
            for j in range(beadBox.parentApp.canvas.assembly.minY, beadBox.parentApp.canvas.assembly.maxY+1):
                loc = beadBox.parentApp.canvas.assembly.GetLoc((i,j))
                if not loc is None:
                    if loc.type == oldName:
                        beadBox.parentApp.canvas.assembly.GetLoc((i,j)).type = newName

    def PickColor(self, event):
        if event.LeftDown() == True:
            box = event.EventObject.Parent
            if SimUtils.globalApp.useAdvancedColorPicker == False:
                dlg = wx.ColourDialog(self)
         
                # Ensure the full colour dialog is displayed, not the abbreviated version.
                dlg.GetColourData().SetChooseFull(True)
         
                if dlg.ShowModal() == wx.ID_OK:
                    data = dlg.GetColourData()
                    box.beadType.color = str(data.GetColour().Get())
                    box.colorButton.SetBitmapLabel(self.GetBeadButtonBitmap(box.beadType.color))
                    box.colorButton.Refresh()
                    SimUtils.globalApp.canvas.Refresh()
                dlg.Destroy()
            else:
                cd = wx.ColourData()
                if type(box.beadType.color) is str:
                    cd.SetColour(wx.TheColourDatabase.Find(box.beadType.color))
                else:
                    cd.SetColour(box.beadType.color)
                dlg = CCD.CubeColourDialog(self, cd)
                if dlg.ShowModal() == wx.ID_OK:
                    colourData = dlg.GetColourData()
                    colorTuple = (colourData.GetColour().Red(), colourData.GetColour().Green(), colourData.GetColour().Blue(), colourData.GetColour().Alpha())
                    box.beadType.color = str(colorTuple)
                    #box.beadType.color = str(colourData.GetColour().Get())
                    box.colorButton.SetBitmapLabel(self.GetBeadButtonBitmap(box.beadType.color))
                    box.colorButton.Refresh()
                    SimUtils.globalApp.canvas.Refresh()
                    self.Refresh()
                dlg.Destroy()

    def GetBeadButtonBitmap(self, color):
        w, h = 20, 20
        bmp = wx.Bitmap(w, h)
        dc = wx.MemoryDC()
        dc.SelectObject(bmp)
        dc.Clear()
        pointBrush = self.parentApp.canvas.ColoredBrush(color)
        dc.SetBrush(pointBrush)
        dc.DrawCircle((w/2,h/2), 10)
        dc.SelectObject(wx.NullBitmap)
        return bmp

#---------------------------------------------------------------------------


class OritatamiSimulatorApp(SimApp):
    def __init__(self):
        self.showMessages = True
        self.messageQueue = []
        self.bShowDebugMsgs = True
        self.showDisplayUpdates = True
        self.useAdvancedColorPicker = False
        SimApp.__init__(self)

    def OnInit(self):

        self.beadTypePanel          = None
        self.seqPanel               = None
        self.beadSequenceWin        = None
        self.messageWin             = None
        self.bpSize                 = (0,0)
        self.systemDefinitionFile   = None
        self.systemDefinitionFileFullPath = None
        self.seedDefinitionFile     = None
        self.assemblyDefinitionFile = None
        self.docFrame               = None
        
        provider = wx.SimpleHelpProvider()
        wx.HelpProvider.Set(provider)

        SimApp.OnInit(self, title="Oritatami Simulator")
        self.frame.SetIcon(wx.Icon(wx.Bitmap("images/logo.png", wx.BITMAP_TYPE_PNG)))
        
        SimUtils.DocumentationFileList = [ "Overview", "System Parameters", "File Formats", "Tips and Tricks"]
    
        #self.AddBeadTypeWindow()
        self.AddSequenceWindow()
        #self.AddMessageWindow()
        
        # tell the manager to 'commit' all the changes just made
        self.frame._mgr.Update()
        #self.frame._mgr.OnFloatingPaneClosed()

        self.frame.Maximize()
        self.frame.Show(True)
        self.canvas.ScrollToCoords((0,0))
        
        return True


    def CreateCanvas(self):
        self.canvas = OritatamiSimCanvas(self.frame, wx.ID_ANY, self.frame, wx.DefaultSize, self)
        self.SetMouseMode(ORIT_MODE_PAN_ZOOM)

    def CreateStatusBar(self):
        self.frame.CreateStatusBar(3)
        self.frame.SetStatusWidths([200,200,-1])

    def CreateMenu(self):
        menuBar = wx.MenuBar()
        menu = wx.Menu()
        itemNew = menu.Append(-1, "New system", "New system")
        self.Bind(wx.EVT_MENU, self.OnNew, itemNew)
        menu.AppendSeparator()
        itemLSys = menu.Append(-1, "Load system definition", "Load system definition")
        self.Bind(wx.EVT_MENU, self.OnLoadSystem, itemLSys)
        item1 = menu.Append(-1, "&Load assembly\tCtrl-L", "Load assembly")
        self.Bind(wx.EVT_MENU, self.OnLoadAssembly, item1)
        itemR = menu.Append(-1, "Reload assembly", "Reload assembly")
        self.Bind(wx.EVT_MENU, self.OnReload, itemR)
        menu.AppendSeparator()
        itemR = menu.Append(-1, "&Reset to seed\tCtrl-R", "Reset system to the seed")
        self.Bind(wx.EVT_MENU, self.OnReset, itemR)
        menu.AppendSeparator()
        itemSBT = menu.Append(-1, "Save system definition", "Save oritatami system definition")
        self.Bind(wx.EVT_MENU, self.OnSaveSystemDef, itemSBT)
        itemSBTA = menu.Append(-1, "Save system definition as..", "Save oritatami system definition as..")
        self.Bind(wx.EVT_MENU, self.OnSaveSystemDefAs, itemSBTA)
        menu.AppendSeparator()
        itemSetSeed = menu.Append(-1, "Set current assembly as seed", "Set current assembly as seed assembly")
        self.Bind(wx.EVT_MENU, self.OnSetSeed, itemSetSeed)
        itemSS = menu.Append(-1, "Save seed", "Save seed assembly")
        self.Bind(wx.EVT_MENU, self.OnSaveSeed, itemSS)
        itemSSA = menu.Append(-1, "Save seed as..", "Save seed assembly as..")
        self.Bind(wx.EVT_MENU, self.OnSaveSeedAs, itemSSA)
        menu.AppendSeparator()
        itemSA = menu.Append(-1, "Save current assembly", "Save current assembly")
        self.Bind(wx.EVT_MENU, self.OnSaveAssembly, itemSA)
        itemSAA = menu.Append(-1, "Save current assembly as..", "Save current assembly as..")
        self.Bind(wx.EVT_MENU, self.OnSaveAssemblyAs, itemSAA)
        menu.AppendSeparator()
        item2 = menu.Append(-1, "&Save system definition and seed\tCtrl-S", "Save system definition and seed assembly")
        self.Bind(wx.EVT_MENU, self.OnSaveAll, item2)
        menu.AppendSeparator()
        itemSVG = menu.Append(-1, "Save assembly as an SVG..")
        self.Bind(wx.EVT_MENU, self.OnSaveAsSVG, itemSVG)
        menu.AppendSeparator()
        item3 = menu.Append(-1, "E&xit\tCtrl-X", "Exit simulator")
        self.Bind(wx.EVT_MENU, self.OnExitApp, item3)
        menuBar.Append(menu, "&File")

        menuCtrl = wx.Menu()
        itemBP = menuCtrl.Append(-1, "&Breakpoints\tCtrl-B", "Set breakpoints")
        self.Bind(wx.EVT_MENU, self.OnBreakpoints, itemBP)
        menuCtrl.AppendSeparator()
        itemStep = menuCtrl.Append(-1, "&Forward\tCtrl-F", "Forward one step")
        self.Bind(wx.EVT_MENU, self.OnForward, itemStep)
        itemFF = menuCtrl.Append(-1, "&Go\tCtrl-G", "Fast forward")
        self.Bind(wx.EVT_MENU, self.OnGo, itemFF)
        itemQuit = menuCtrl.Append(-1, "&Quit\tCtrl-Q", "Quit fast forward")
        self.Bind(wx.EVT_MENU, self.OnQuit, itemQuit)
        menuBar.Append(menuCtrl, "&Control")

        self.menuView = wx.Menu()
        self.itemShowMess = self.menuView.Append(-1, "Show message window", "Toggle display of message window", kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.OnShowMessageWin, self.itemShowMess)
        self.menuView.Check(self.itemShowMess.GetId(), True)
        self.itemShowBTEdit = self.menuView.Append(-1, "Show bead type editor window", "Toggle display of bead type editor window", kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.OnShowBTEditWin, self.itemShowBTEdit)
        self.menuView.Check(self.itemShowBTEdit.GetId(), False)
        self.itemShowSeq = self.menuView.Append(-1, "Show bead sequence window", "Toggle display of bead sequence window", kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.OnShowSequenceWin, self.itemShowSeq)
        self.menuView.Check(self.itemShowSeq.GetId(), True)
        self.menuView.AppendSeparator()
        self.itemShowDebug = self.menuView.Append(-1, "Show debugging messages\tCtrl-Z", "Toggle display of debugging messages", kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.OnShowDebugMsgs, self.itemShowDebug)
        self.menuView.Check(self.itemShowDebug.GetId(), self.bShowDebugMsgs)
        self.menuView.AppendSeparator()
        itemSTC = self.menuView.Append(-1, "&Center\tCtrl+C", "Scroll to center")
        self.Bind(wx.EVT_MENU, self.OnCenter, itemSTC)
        self.menuView.AppendSeparator()
        itemZIn = self.menuView.Append(-1, "Zoom in\tCtrl-+", "Zoom in")
        self.Bind(wx.EVT_MENU, self.OnZoomIn, itemZIn)
        itemZOut = self.menuView.Append(-1, "Zoom out\tCtrl--", "Zoom out")
        self.Bind(wx.EVT_MENU, self.OnZoomOut, itemZOut)
        self.menuView.AppendSeparator()
        itemTG = self.menuView.Append(-1, "&Toggle grid lines\tCtrl-T", "Toggle display of grid lines")
        self.Bind(wx.EVT_MENU, self.OnToggleGrid, itemTG)
        self.itemTM = self.menuView.Append(-1, "Update &messages\tCtrl-M", "Toggle display of messages", kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.OnMessageToggle, self.itemTM)
        self.menuView.Check(self.itemTM.GetId(), True)
        self.itemTD = self.menuView.Append(-1, "Update &display\tCtrl-D", "Toggle display of assembly updates", kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.OnDisplayToggle, self.itemTD)
        self.menuView.Check(self.itemTD.GetId(), True)
        self.menuView.AppendSeparator()
        itemRefresh = self.menuView.Append(-1, "Refresh screen\tF2", "Refresh screen")
        self.Bind(wx.EVT_MENU, self.OnRefresh, itemRefresh)
        menuBar.Append(self.menuView, "&View")

        self.menuSettings = wx.Menu()
        itemSetDL = self.menuSettings.Append(-1, "Set delay length", "Set delay length system parameter")
        self.Bind(wx.EVT_MENU, self.OnSetDelayLength, itemSetDL)
        itemSetArity = self.menuSettings.Append(-1, "Set arity", "Set arity system parameter")
        self.Bind(wx.EVT_MENU, self.OnSetArity, itemSetArity)
        self.itemSetLWB = self.menuSettings.Append(-1, "Lock without binding", "Set lock without binding system parameter", kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.OnSetLockWithoutBinding, self.itemSetLWB)
        self.menuSettings.Check(self.itemSetLWB.GetId(), False)
        self.itemSetBFFS = self.menuSettings.Append(-1, "Bind full floppy segment", "Set bind full floppy segment system parameter", kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.OnSetBindFullFloppySegment, self.itemSetBFFS)
        self.menuSettings.Check(self.itemSetBFFS.GetId(), False)
        itemSetMFL = self.menuSettings.Append(-1, "Set max floppy length", "Set maximum floppy length to simulate")
        self.Bind(wx.EVT_MENU, self.OnSetMaxFloppyLength, itemSetMFL)
        self.menuSettings.AppendSeparator()
        self.itemAdvColor = self.menuSettings.Append(-1, "Use advanced color picker", "Use advanced color picker dialog", kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.OnAdvancedColorChecked, self.itemAdvColor)
        self.menuSettings.Check(self.itemAdvColor.GetId(), False)
        menuBar.Append(self.menuSettings, "&Settings")
    
        menu2 = wx.Menu()
        itemDoc = menu2.Append(-1, "Documentation", "Show documentation")
        self.Bind(wx.EVT_MENU, self.OnDoc, itemDoc)
        itemH = menu2.Append(-1, "&Help\tCtrl-H", "Show help")
        self.Bind(wx.EVT_MENU, self.OnHelp, itemH)
        menuBar.Append(menu2, "&Help")
        self.frame.SetMenuBar(menuBar)
        
    def CreateToolbar(self):
        tb1 = wx.ToolBar(self.frame, -1, wx.DefaultPosition, wx.DefaultSize, wx.TB_DOCKABLE | wx.TB_FLAT)
        tb1.SetToolSeparation(20)
        tb1.SetToolBitmapSize(wx.Size(24,24))
        tb1.AddTool(ORIT_ID_NEW, "New System", wx.Image("images/new.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="New System")#, longHelp="New oritatami system")
        tb1.AddTool(ORIT_ID_LOAD, "Load System", wx.Image("images/load.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Load System")#, longHelp="Load oritatami system")
        tb1.AddTool(ORIT_ID_SAVE, "Save System", wx.Image("images/save.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Save System")#, longHelp="Save oritatami system")
        tb1.AddTool(ORIT_ID_RESET, "Reset System", wx.Image("images/reset.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Reset System")#, longHelp="Reset oritatami system")
        tb1.Realize()
        tb1.Bind(wx.EVT_TOOL, self.OnTool)

        self.frame._mgr.AddPane(tb1, wx.aui.AuiPaneInfo().
                          Name("File toolbar").Caption("File Toolbar").
                          ToolbarPane().Top())
        #tb1.AddSeparator()
        #tb1.AddSeparator()
        tb1 = wx.ToolBar(self.frame, -1, wx.DefaultPosition, wx.DefaultSize, wx.TB_DOCKABLE | wx.TB_FLAT)
        tb1.SetToolSeparation(20)
        tb1.SetToolBitmapSize(wx.Size(24,24))
        tb1.AddTool(ORIT_ID_FR, "Fast Rewind", wx.Image("images/fastleft.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Fast Rewind")#, longHelp="Fast rewind")
        tb1.AddTool(ORIT_ID_REV, "Rewind", wx.Image("images/left.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Rewind")#, longHelp="Rewind one step")
        tb1.AddTool(ORIT_ID_STOP, "Pause", wx.Image("images/pause.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Pause")#, longHelp="Pause simulation")
        tb1.AddTool(ORIT_ID_FOR, "Forward", wx.Image("images/right.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Forward")#, longHelp="Forward one step")
        tb1.AddTool(ORIT_ID_FF, "Fast Forward", wx.Image("images/fastright.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Fast Forward")#, longHelp="Fast forward")
        tb1.AddSeparator()
        tb1.AddTool(ORIT_ID_ZOOMIN, "Zoom In", wx.Image("images/zoomin.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Zoom In")#, longHelp="Zoom in")
        tb1.AddTool(ORIT_ID_ZOOMOUT, "Zoom Out", wx.Image("images/zoomout.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Zoom Out")#, longHelp="Zoom out")
        tb1.Realize()
        tb1.Bind(wx.EVT_TOOL, self.OnTool)

        self.frame._mgr.AddPane(tb1, wx.aui.AuiPaneInfo().
                          Name("Simulation toolbar").Caption("Simulation Toolbar").
                          ToolbarPane().Top())
        tb1 = wx.ToolBar(self.frame, -1, wx.DefaultPosition, wx.DefaultSize, wx.TB_DOCKABLE | wx.TB_FLAT)
        tb1.SetToolSeparation(10)
        #tb1.SetToolPacking(10)
        #tb1.SetToolBitmapSize(wx.Size(16,16))
        btnImg = wx.Image("images/pan-zoom.png", wx.BITMAP_TYPE_PNG)
        btnImg.Rescale(45, 25)
        tb1.AddRadioTool(ORIT_ID_MODE_PAN_ZOOM, "Pan/Zoom", btnImg.ConvertToBitmap(), shortHelp="Pan (left mouse)/zoom (right mouse)", longHelp="Pan (left mouse) and zoom (right mouse)")
        btnImg = wx.Image("images/add-remove-bead.png", wx.BITMAP_TYPE_PNG)
        btnImg.Rescale(25, 15)
        tb1.AddRadioTool(ORIT_ID_MODE_DRAW_BEAD, "Add/Remove Beads", btnImg.ConvertToBitmap(), shortHelp="Add beads (left mouse)/remove beads (right mouse)", longHelp="Add beads (left mouse) and remove beads (right mouse)")
        btnImg = wx.Image("images/add-remove-link.png", wx.BITMAP_TYPE_PNG)
        btnImg.Rescale(60, 15)
        tb1.AddRadioTool(ORIT_ID_MODE_DRAW_LINK, "Add/Remove Links", btnImg.ConvertToBitmap(), shortHelp="Add links (left mouse)/remove links (right mouse)", longHelp="Add links (left mouse)/remove links (right mouse)")
        btnImg = wx.Image("images/add-remove-bond.png", wx.BITMAP_TYPE_PNG)
        btnImg.Rescale(60, 15)
        tb1.AddRadioTool(ORIT_ID_MODE_DRAW_BOND, "Add/Remove Bonds", btnImg.ConvertToBitmap(), shortHelp="Add bonds (left mouse)/remove bonds (right mouse)", longHelp="Add bonds (left mouse)/remove bonds (right mouse)")
        tb1.Realize()
        tb1.Bind(wx.EVT_TOOL, self.OnTool)

        self.frame._mgr.AddPane(tb1, wx.aui.AuiPaneInfo().
                          Name("Mouse mode toolbar").Caption("Mouse Mode Toolbar").
                          ToolbarPane().Top())

    def OnTool(self, evt):
        if evt.GetId() == ORIT_ID_MODE_PAN_ZOOM:
            self.SetMouseMode(ORIT_ID_MODE_PAN_ZOOM)
        elif evt.GetId() == ORIT_ID_MODE_DRAW_BEAD:
            self.SetMouseMode(ORIT_ID_MODE_DRAW_BEAD)
        elif evt.GetId() == ORIT_ID_MODE_DRAW_LINK:
            self.SetMouseMode(ORIT_ID_MODE_DRAW_LINK)
        elif evt.GetId() == ORIT_ID_MODE_DRAW_BOND:
            self.SetMouseMode(ORIT_ID_MODE_DRAW_BOND)
        elif evt.GetId() == ORIT_ID_FR:
            self.ShowErrorDialog("Sorry, this is not implemented yet")
        elif evt.GetId() == ORIT_ID_REV:
            self.ShowErrorDialog("Sorry, this is not implemented yet")
        elif evt.GetId() == ORIT_ID_STOP:
            self.canvas.DoQuit()
        elif evt.GetId() == ORIT_ID_FOR:
            self.canvas.DoForwardStep()
        elif evt.GetId() == ORIT_ID_FF:
            self.canvas.DoGo()
        elif evt.GetId() == ORIT_ID_NEW:
            self.canvas.DoNew()
        elif evt.GetId() == ORIT_ID_LOAD:
            self.canvas.DoLoad(bAssembly=True, bSystem=True)
        elif evt.GetId() == ORIT_ID_RESET:
            self.canvas.Reset()
        elif evt.GetId() == ORIT_ID_SAVE:
            self.canvas.DoSaveAll()
        elif evt.GetId() == ORIT_ID_ZOOMIN:
            self.canvas.ZoomThenRecenter(True)
        elif evt.GetId() == ORIT_ID_ZOOMOUT:
            self.canvas.ZoomThenRecenter(False)
        
    def OnShowMessageWin(self, event):
        pane = self.frame._mgr.GetPane(self.ctrlPanel)
        if self.itemShowMess.IsChecked():
            pane.Show()
        else:
            pane.Hide()
        self.frame._mgr.Update()
        
    def OnSetCurrentBead(self, event):
        if event.LeftDown():
            if not self.beadSequenceWin.HasSelection():
                SimUtils.globalApp.ShowErrorDialog("You must select a bead name to make it the current location")
                return
            else:
                selRange = self.beadSequenceWin.GetSelectionRange()
                selStart = selRange.GetStart()
                selEnd = selRange.GetEnd()
                selText = self.beadSequenceWin.GetBuffer().GetTextForRange((selStart, selEnd-1))
                if "," in selText:
                    SimUtils.globalApp.ShowErrorDialog("You must select only a single bead name to make it the current location")
                    return
                
                # Compare string in window against sequence currently being used and don't allow if different
                cycleStr = self.GetStringFromBeadCycle()
                winStr = str(self.beadSequenceWin.GetValue())
                if not cycleStr == winStr:
                    SimUtils.globalApp.ShowErrorDialog("The bead cycle currently in the text box differs from that used by the\nsimulator. Press 'Apply' to make it current or 'Reset' to revert first.")
                    return

                pos   = 0
                count = 0
                for char in cycleStr:
                    if pos == selStart:
                        break
                    if char == ',':
                        count = count + 1
                    pos = pos + 1
                
                # Looks good, update the current position and redraw the text
                self.canvas.assembly.currCyclePos = count
                self.DrawBeadCycle()
                self.canvas.SetFocus()

    def GetStringFromBeadCycle(self):
        outStr = ""
        for x in self.canvas.assembly.beadCycle:
            if len(outStr) > 0:
                outStr = outStr + ","
            outStr = outStr + x
        return outStr

    def OnSequenceApply(self, event):
        if event.LeftDown():
            cycleStr = self.GetStringFromBeadCycle()
            winStr = str(self.beadSequenceWin.GetValue()).strip()
            if not cycleStr == winStr:
                winArr = winStr.split(",")
                notDefined = []
                for bt in winArr:
                    if " " in bt:
                        SimUtils.globalApp.ShowErrorDialog("Bead type '" + bt + "' is invalid.  Spaces are not allowed.")
                        return
                    elif not bt in self.canvas.assembly.ONodeTypes:
                        notDefined.append(bt)
                if len(notDefined) > 0:
                    if len(notDefined) == 1:
                        SimUtils.globalApp.ShowErrorDialog("Bead type " + str(notDefined[0]) + " is not defined.")
                    else:
                        SimUtils.globalApp.ShowErrorDialog("Bead types " + str(notDefined) + " are not defined.")
                    return
                self.canvas.assembly.beadCycle = winArr
                SimUtils.globalApp.OutputMessage("The primary structure has been set to " + str(self.canvas.assembly.beadCycle))
                self.canvas.SetFocus()
        
    def OnSequenceReset(self, event):
        if event.LeftDown():
            #dlg = ResetBeadCycleDialog(self.beadSequenceWin, wx.ID_ANY, "Are you sure you want to reset the sequence and discard your changes?", caption="Confirm reset")
            dlg = ResetBeadCycleDialog(self.beadSequenceWin, wx.ID_ANY)
            dlg.CenterOnScreen()
            retId = dlg.ShowModal()
            if retId == ORIT_ID_RESET_BEADSEQ:
                dlg.Destroy()
                self.canvas.assembly.currCyclePos = 0
                self.DrawBeadCycle()
            elif retId == ORIT_ID_RELOAD_BEADSEQ:
                dlg.Destroy()
                self.canvas.assembly.beadCycle = self.canvas.assembly.origBeadCycle
                self.canvas.assembly.currCyclePos = 0
                self.DrawBeadCycle()
            else:
                dlg.Destroy()
            self.canvas.SetFocus()

    def SetMouseMode(self, mode):
        if not mode == self.canvas.mouseMode:
            self.canvas.linkStart = None
            self.canvas.linkEnd = None
            if (self.canvas.posSelected is not None) or (self.canvas.negSelected is not None):
                self.canvas.posSelected = None
                self.canvas.negSelected = None
                self.canvas.Refresh()
        if mode == ORIT_ID_MODE_PAN_ZOOM:
            self.canvas.mouseMode = ORIT_MODE_PAN_ZOOM
        elif mode == ORIT_ID_MODE_DRAW_BEAD:
            self.canvas.mouseMode = ORIT_MODE_DRAW_BEAD
            if self.beadTypePanel is None:
                self.AddBeadTypeWindow()
            pane = self.frame._mgr.GetPane(self.beadTypePanel)
            self.menuView.Check(self.itemShowBTEdit.GetId(), True)
            pane.Show()
            self.frame._mgr.Update()
        elif mode == ORIT_ID_MODE_DRAW_LINK:
            self.canvas.mouseMode = ORIT_MODE_DRAW_LINK
        elif mode == ORIT_ID_MODE_DRAW_BOND:
            self.canvas.mouseMode = ORIT_MODE_DRAW_BOND

    #===========================================================================
    # def SetOutputMessage(self, msg):
    #     if not self.showMessages:
    #         self.messageQueue.append(msg)
    #         return
    #     self.messageWin.SetInsertionPointEnd()
    #     self.messageWin.WriteText("\n" + msg)
    #     self.messageWin.SetInsertionPointEnd()
    #     self.messageWin.ScrollLines(self.messageWin.GetNumberOfLines()*5)
    #===========================================================================

    def DrawBeadCycle(self):
        if (not self.showMessages):# or (self.beadSequenceWin == None):
            return
        
        self.beadSequenceWin.Freeze()
        self.beadSequenceWin.Clear()
        #outStr = str(self.canvas.assembly.beadCycle[0:self.canvas.assembly.currCyclePos]) + "X" + str(self.canvas.assembly.beadCycle[self.canvas.assembly.currCyclePos:])
        
        outStr = ""
        pos = 0
        for x in self.canvas.assembly.beadCycle:
            if pos == self.canvas.assembly.currCyclePos:
                if len(outStr) > 0:
                    self.beadSequenceWin.WriteText(outStr + ",")

                self.beadSequenceWin.BeginBold()
                self.beadSequenceWin.WriteText(x)
                self.beadSequenceWin.EndBold()
                outStr = ""
            elif pos > 0:
                outStr = outStr + "," + x
            else:
                outStr = outStr + x
            pos = pos + 1
        self.beadSequenceWin.WriteText(outStr)
        #self.beadSequenceWin.BeginFontSize(14)
        #self.beadSequenceWin.SetInsertionPointEnd()
        #self.beadSequenceWin.WriteText(outStr)
        #self.beadSequenceWin.SetInsertionPointEnd()
        #self.beadSequenceWin.EndFontSize()
        self.beadSequenceWin.Thaw()
        #self.beadSequenceWin.ScrollLines(self.beadSequenceWin.GetNumberOfLines()*5)

    def AddSequenceWindow(self):
        self.seqPanel = wx.Panel(self.frame, wx.ID_ANY, pos=wx.DefaultPosition, size=(300,100), style = wx.TAB_TRAVERSAL)
        
        sz = wx.BoxSizer(wx.HORIZONTAL)
        leftSizer = wx.BoxSizer(wx.VERTICAL)
        self.applyButton  = wx.Button(self.seqPanel, wx.ID_ANY, "Apply", size=(55,25))
        self.applyButton.Bind(wx.EVT_MOUSE_EVENTS, self.OnSequenceApply)
        leftSizer.Add(self.applyButton)
        self.setCurrButton  = wx.Button(self.seqPanel, wx.ID_ANY, "Current", size=(55,25))
        self.setCurrButton.Bind(wx.EVT_MOUSE_EVENTS, self.OnSetCurrentBead)
        #MJPTODO: figure out how to display mouse over text
        self.setCurrButton.SetHelpText("MJPTODO: add help text")
        leftSizer.Add(self.setCurrButton)
        self.resetButton  = wx.Button(self.seqPanel, wx.ID_ANY, "Reset", size=(55,25))
        self.resetButton.Bind(wx.EVT_MOUSE_EVENTS, self.OnSequenceReset)
        leftSizer.Add(self.resetButton)
        sz.Add(leftSizer)
    
        self.beadSequenceWin = rt.RichTextCtrl(self.seqPanel, wx.ID_ANY, pos=wx.DefaultPosition, size=(500,200))#, style=wx.NOSCROLL|wx.HSCROLL|wx.NO_BORDER)
        #self.beadSequenceWin.SetEditable(False)
        sz.Add(self.beadSequenceWin, 20, wx.EXPAND, 20)
        self.seqPanel.SetSizerAndFit(sz)
        self.frame._mgr.Bind(wx.aui.EVT_AUI_PANE_CLOSE, self.OnCloseSequencePane)
        self.frame._mgr.AddPane(self.seqPanel, wx.BOTTOM, 'Bead Sequence Panel')
        self.frame._mgr.Update()

    def AddBeadTypeWindow(self):
        # Destroy the existing copy of the window
        if not self.beadTypePanel is None:
            pane = self.frame._mgr.GetPane(self.beadTypePanel)
            pane.Hide()
            self.frame._mgr.DetachPane(self.beadTypePanel)
            self.frame._mgr.Update()
            #for bb in self.beadBoxes:
            #    self.beadBoxes[bb].Destroy()
            self.beadTypePanel.Destroy()
            self.beadTypePanel = None
        self.beadTypePanel = scrolled.ScrolledPanel(self.frame, wx.ID_ANY, pos=wx.DefaultPosition, size=(400,150), style = wx.TAB_TRAVERSAL)
        beadTypePanelSizer = wx.GridSizer(cols=1, hgap=5, vgap=5)
        
        # If the bead set size is greater than 250, don't load it and output a message saying it's too big to display
        bTooBig = (len(self.canvas.assembly.ONodeTypes) > 250)
        if bTooBig == False:
            self.beadBoxes = dict()
            bb = OritatamiBeadBox(None, self.beadTypePanel, beadTypePanelSizer, self, self.beadBoxes)
            self.beadBoxes[""] = bb
    
            for beadTypeName in self.canvas.assembly.ONodeTypes:
                beadType = self.canvas.assembly.ONodeTypes[beadTypeName]
                bb = OritatamiBeadBox(beadType, self.beadTypePanel, beadTypePanelSizer, self, self.beadBoxes)
                self.beadBoxes[beadTypeName] = bb
            beadTypePanelSizer.Layout()
        
        else:
            self.OutputMessage("The bead type set is too large (greater than 250) to be loaded in the bead type window.")
            self.ShowWarningDialog("The bead type set is too large (greater than 250) to be loaded in the bead type window.")
        
        self.beadTypePanel.SetSizerAndFit(beadTypePanelSizer)
        
        self.beadTypePanel.Bind(wx.EVT_MOUSE_EVENTS, self.OnBeadPanelClick)
        self.beadTypePanel.Bind(wx.EVT_SIZE, self.OnBeadPanelResize)
        self.frame._mgr.Bind(wx.aui.EVT_AUI_PANE_CLOSE, self.OnCloseBTEditPane)
        self.frame._mgr.AddPane(self.beadTypePanel, wx.LEFT, 'Bead Type Window')
        self.frame._mgr.Update()
        self.beadTypePanel.SetupScrolling()
        self.bpSize = self.beadTypePanel.GetSize()
        
        if bTooBig == True:
            #self.beadTypePanel.Hide()
            pane = self.frame._mgr.GetPane(self.beadTypePanel)
            pane.Hide()
            self.menuView.Check(self.itemShowBTEdit.GetId(), False)
        else:
            self.menuView.Check(self.itemShowBTEdit.GetId(), True)

    def OnBeadTypeDelete(self, event):
        if event.LeftDown():
            box = event.EventObject.Parent
            bname = str(box.editName.GetValue())
            dlg = wx.MessageDialog(self.beadTypePanel, "Are you sure you want to delete bead type '" + bname + "'?", caption="Confirm delete")
            dlgRet = dlg.ShowModal()
            dlg.Destroy()
            if dlgRet == wx.ID_OK:
                beadType = box.beadType

                # Delete the bead type from the assembly
                removeList = []
                for i in range(self.canvas.assembly.minX, self.canvas.assembly.maxX+1):
                    for j in range(self.canvas.assembly.minY, self.canvas.assembly.maxY+1):
                        loc = self.canvas.assembly.GetLoc((i, j))
                        if not loc is None:
                            if loc.type == beadType.name:
                                self.canvas.assembly.RemoveNode((i, j))
                                removeList.append((i, j))
                self.canvas._lock.acquire()
                self.canvas.RemoveQueue = self.canvas.RemoveQueue + removeList
                self.canvas._lock.release()
                self.canvas.DrawUpdate(self.canvas.pdc)
                self.canvas.Refresh()
                
                # Delete the bead type from the bead cycle
                newBeadCycle = []
                bDidDelete = False
                for b in self.canvas.assembly.beadCycle:
                    if not b == beadType.name:
                        newBeadCycle.append(b)
                        bDidDelete = True
                if bDidDelete == True:
                    self.canvas.assembly.beadCycle = newBeadCycle
                    self.canvas.assembly.InitializeFloppySegment()
                    self.DrawBeadCycle()
                
                # Delete the bead type box from the editor
                panelSizer = box.parentPanelSizer
                bbSizer = box.beadBoxSizer
                panelSizer.Hide(bbSizer)
                panelSizer.Remove(bbSizer)
                panelSizer.Layout()
                
                # Delete the bead type from the simulator's set
                del self.canvas.assembly.ONodeTypes[beadType.name]
        
    def OnBeadPanelResize(self, event):
        bp = event.EventObject
        w,h = bp.GetSize()
        if not (w,h) == self.bpSize:
            self.bpSize = (w,h)
            rowMod = w % 200
            numCols = int(w / 200)
            if rowMod > 50:
                numCols = numCols + 1
            if numCols < 1:
                numCols = 1
            #print("old size: " + str(self.bpSize) + ", new size: " + str((w,h)) + ", new num cols: " + str(numCols))
            sizer = bp.GetSizer()
            sizer.SetCols(numCols)
            #sizer.Layout()
            #self.frame._mgr.Update()
            
        
    def OnBeadPanelClickStatText(self, event):
        if event.LeftDown() == True:
            box = event.EventObject.Parent
            self.SelectCurrentDrawingBead(box)

    def OnBeadPanelClick(self, event):
        if event.LeftDown() == True:
            x = event.GetX()
            y = event.GetY()
            children = self.beadTypePanel.GetChildren()
            for child in children:
                rect = child.GetRect()
                if (x >= rect[0]) and (x <= (rect[0]+rect[2])) and (y >= rect[1]) and (y <= (rect[1]+rect[3])):
                    self.SelectCurrentDrawingBead(child)
        
    def SelectCurrentDrawingBead(self, clickedBox):
        if clickedBox.beadType.name == "":
            return
        if clickedBox.beadType == self.canvas.drawingModeBeadType:
            for bt in clickedBox.beadType.bondToTypes:
                self.beadBoxes[bt].SetBackgroundColour(wx.NullColour)
                self.beadBoxes[bt].Refresh()
            self.canvas.drawingModeBeadType = None
            clickedBox.SetBackgroundColour(wx.NullColour)
        else:
            if self.canvas.drawingModeBeadType is not None:
                self.beadBoxes[self.canvas.drawingModeBeadType.name].SetBackgroundColour(wx.NullColour)
                self.beadBoxes[self.canvas.drawingModeBeadType.name].Refresh()
                for bt in self.canvas.drawingModeBeadType.bondToTypes:
                    self.beadBoxes[bt].SetBackgroundColour(wx.NullColour)
                    self.beadBoxes[bt].Refresh()
            for bt in clickedBox.beadType.bondToTypes:
                self.beadBoxes[bt].SetBackgroundColour("Yellow")
                self.beadBoxes[bt].Refresh()
            self.canvas.drawingModeBeadType = clickedBox.beadType
            clickedBox.SetBackgroundColour("Grey")
        clickedBox.Refresh()
        
    def OnCloseSequencePane(self, event):
        if event.GetPane().window == self.seqPanel:
            self.menuView.Check(self.itemShowSeq.GetId(), False)
        event.Skip()
        
    def OnShowSequenceWin(self, event):
        if self.seqPanel is None:
            self.AddSequenceWindow()
        pane = self.frame._mgr.GetPane(self.seqPanel)
        if self.itemShowSeq.IsChecked():
            pane.Show()
        else:
            pane.Hide()
        self.frame._mgr.Update()
        
    def OnShowDebugMsgs(self, event):
        self.canvas.ToggleDebugMsgs()
        
    def OnCloseBTEditPane(self, event):
        if event.GetPane().window == self.beadTypePanel:
            self.menuView.Check(self.itemShowBTEdit.GetId(), False)
        event.Skip()
        
    def OnShowBTEditWin(self, event):
        if self.beadTypePanel is None:
            self.AddBeadTypeWindow()
        pane = self.frame._mgr.GetPane(self.beadTypePanel)
        if self.itemShowBTEdit.IsChecked():
            pane.Show()
        else:
            pane.Hide()
        self.frame._mgr.Update()
        
    def OnSetDelayLength(self, event):
        dlg = wx.TextEntryDialog(self.frame, "Delay length:", caption="Set delay length value", value=str(self.canvas.assembly.delayLength))
        if dlg.ShowModal() == wx.ID_OK:
            try:
                self.canvas.assembly.delayLength = int(dlg.GetValue())
            except ValueError:
                self.ShowErrorDialog("Delay length must be an integer value.")
        dlg.Destroy()
        
    def OnSetMaxFloppyLength(self, event):
        dlg = wx.TextEntryDialog(self.frame, "Maximum floppy length:", caption="Set maximum floppy length value", value=str(self.canvas.assembly.maxFloppyLength))
        if dlg.ShowModal() == wx.ID_OK:
            try:
                self.canvas.assembly.maxFloppyLength = int(dlg.GetValue())
            except ValueError:
                self.ShowErrorDialog("Maximum floppy length must be an integer value.")
        dlg.Destroy()

    def OnSetArity(self, event):
        dlg = wx.TextEntryDialog(self.frame, "Arity:", caption="Set arity value", value=str(self.canvas.assembly.arity))
        if dlg.ShowModal() == wx.ID_OK:
            try:
                self.canvas.assembly.arity = int(dlg.GetValue())
            except ValueError:
                self.ShowErrorDialog("Arity must be an integer value.")
        dlg.Destroy()
        
    def OnSetLockWithoutBinding(self, event):
        if self.itemSetLWB.IsChecked():
            self.canvas.assembly.lockWithoutBinding = True
        else:
            self.canvas.assembly.lockWithoutBinding = False
            
    def OnSetBindFullFloppySegment(self, event):
        if self.itemSetBFFS.IsChecked():
            self.canvas.assembly.bindFullFloppySegment = True
        else:
            self.canvas.assembly.bindFullFloppySegment = False
        
    def OnAdvancedColorChecked(self, event):
        if self.itemShowMess.IsChecked():
            self.useAdvancedColorPicker = True
        else:
            self.useAdvancedColorPicker = False

    def ResetBreakpoints(self):
        if len(self.canvas.breakpoints) > 0:
            self.canvas.currentBreakpoint = 0

    def OnNew(self, evt):
        self.canvas.DoNew()

    def OnLoadAssembly(self, evt):
        self.canvas.DoLoad(bAssembly=True, bSystem=False)
        
    def OnLoadSystem(self, evt):
        self.canvas.DoLoad(bAssembly=False, bSystem=True)

    def OnReload(self, evt):
        self.canvas.Reload()
        
    def OnReset(self, evt):
        self.canvas.Reset()

    def OnSaveSystemDef(self, event):
        self.canvas.DoSaveSystemDef()

    def OnSaveSystemDefAs(self, event):
        self.canvas.DoSaveSystemDef(bSaveAs=True)

    def OnSetSeed(self, event):
        self.canvas.DoSetSeed()

    def OnSaveSeed(self, event):
        self.canvas.DoSaveAssembly(bOnlySeed=True)

    def OnSaveSeedAs(self, event):
        self.canvas.DoSaveAssembly(bOnlySeed=True, bSaveAs=True)

    def OnSaveAssembly(self, event):
        self.canvas.DoSaveAssembly()

    def OnSaveAssemblyAs(self, event):
        self.canvas.DoSaveAssembly(bOnlySeed=False, bSaveAs=True)

    def OnSaveAll(self, event):
        self.canvas.DoSaveAll()
        
    def OnSaveAsSVG(self, event):
        self.canvas.DoSaveAsSVG()

    def OnBreakpoints(self, event):
        self.canvas.SetBreakpoints()

    def OnForward(self, event):
        self.canvas.DoForwardStep()
        
    def OnGo(self, event):
        self.canvas.DoGo()
        
    def OnCenter(self, event):
        self.canvas.ScrollToCoords((0,0))
        
    def OnZoomIn(self, event):
        self.canvas.ZoomThenRecenter(True)

    def OnZoomOut(self, event):
        self.canvas.ZoomThenRecenter(False)
    
    def OnMessageToggle(self, event):
        if self.itemTM.IsChecked():
            self.canvas.MessageToggle(True)
        else:
            self.canvas.MessageToggle(False)

    def OnToggleGrid(self, event):
        self.canvas.ToggleGrid()

    def OnDisplayToggle(self, event):
        if self.itemTD.IsChecked():
            self.canvas.DisplayToggle(SIMFW_FULLDISPLAY)
        else:
            self.canvas.DisplayToggle(SIMFW_NODISPLAY)
            
    def OnDoc(self, event):
        if self.docFrame is None:
            self.docFrame = SimUtils.DocumentationFrame(self)
        self.docFrame.SetFocus()

    def OnHelp(self, event):
        #info = wx.AboutDialogInfo()
        #info.SetName("Oritatami Simulator")
        #info.SetVersion("0.1.0")
        helpStr           = "                           Oritatami Simulator\n"
        helpStr = helpStr + "                               version 0.2.0\n\n"
        helpStr = helpStr + "    List of keyboard shortcuts:\n\n"
        helpStr = helpStr + "      l : ('load') select oritatami system definition file to load\n"
        helpStr = helpStr + "      r : ('reset') reset current system back to seed assembly\n"
        helpStr = helpStr + "      s : ('save') save current system\n"
        helpStr = helpStr + "      x : ('exit') exit the simulator\n"
        helpStr = helpStr + "      b : ('breakpoints') set breakpoints\n"
        helpStr = helpStr + "      f : ('forward') simulate one step\n"
        helpStr = helpStr + "      g : ('go') begin fast forward simulation\n"
        helpStr = helpStr + "      q : ('quit') quit fast forward simulation\n"
        helpStr = helpStr + "      z : ('debug') show debugging messages\n"
        helpStr = helpStr + "      c : ('center') center the grid\n"
        helpStr = helpStr + "      +/-: zoom in/out\n"
        helpStr = helpStr + "      t : ('toggle') toggle display of grid lines\n"
        helpStr = helpStr + "      m : ('messages') toggle display of output messages\n"
        helpStr = helpStr + "      d : ('display') toggle updates to the display of the assembly\n"
        helpStr = helpStr + "      F2 : ('refresh'): refresh the assembly display\n"
        helpStr = helpStr + "      <left mouse click>:  grab and pan display\n"
        helpStr = helpStr + "      <right mouse click>:  zoom into clicked location\n"
        helpStr = helpStr + "      Ctrl-<right mouse click>:  zoom out of clicked location\n"
        helpStr = helpStr + "      h : ('help') show this help dialog"
        helpStr = helpStr + "\n\n"
        #info.SetDescription(helpStr)
        #wx.AboutBox(info)
        helpPanel = wx.Panel(self.frame)
        helpText = wx.StaticText(helpPanel, label=helpStr)
        helpText.SetBackgroundColour((233,233,233))
        # add the panes to the manager
        self.frame._mgr.AddPane(helpText, wx.aui.AuiPaneInfo().Float().Caption("Oritatami Simulator Help"))
        self.frame._mgr.Update()
        
    def OnCloseFrame(self, evt):
        # Make sure the simulation thread is killed
        self.canvas.DoQuit()
        timer = wx.Timer(self)
        timer.Start(1000)
            #self.canvas.goThread.exit()

        if not self.docFrame is None:
            wx.PostEvent(self.docFrame, evt)

        # deinitialize the frame manager
        self.frame._mgr.UnInit()
        # delete the frame
        self.frame.Destroy()

#----------------------------------------------------------------------------

class ResetBeadCycleDialog(wx.Dialog):
    def __init__(
            self, parent, ID, size=wx.DefaultSize, pos=wx.DefaultPosition, style=wx.DEFAULT_DIALOG_STYLE):

        title = "Confirm bead sequence reset"
        wx.Dialog.__init__(self, parent, ID, title, pos, size, style)
        sizer = wx.BoxSizer(wx.VERTICAL)

        label = wx.StaticText(self, -1, "Are you sure you want to reset the sequence and\ndiscard your changes?\nChoose 'Reset' to reset to last applied sequence\nChoose 'Reload' to reload original sequence\n")
        sizer.Add(label, 0, wx.ALIGN_CENTRE|wx.ALL, 5)

        btnsizer = wx.StdDialogButtonSizer()
        
        
        self.resetBtn = wx.Button(self, wx.ID_YES, "Reset")
        self.resetBtn.Bind(wx.EVT_MOUSE_EVENTS, self.OnDlgButton)
        #self.resetBtn.SetDefault()
        btnsizer.AddButton(self.resetBtn)

        self.reloadBtn = wx.Button(self, wx.ID_NO, "Reload")
        self.reloadBtn.Bind(wx.EVT_MOUSE_EVENTS, self.OnDlgButton)
        btnsizer.AddButton(self.reloadBtn)
        
        self.cancelBtn = wx.Button(self, wx.ID_APPLY, "Cancel")
        self.cancelBtn.Bind(wx.EVT_MOUSE_EVENTS, self.OnDlgButton)
        btnsizer.AddButton(self.cancelBtn)

        btnsizer.Realize()

        sizer.Add(btnsizer, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT|wx.ALL, 5)

        self.SetSizer(sizer)
        sizer.Fit(self)
        
    def OnDlgButton(self, event):
        if event.LeftDown():
            if event.EventObject == self.resetBtn:
                self.EndModal(ORIT_ID_RESET_BEADSEQ)
            if event.EventObject == self.reloadBtn:
                self.EndModal(ORIT_ID_RELOAD_BEADSEQ)
            if event.EventObject == self.cancelBtn:
                self.EndModal(ORIT_ID_CANCEL_BEADSEQ)

#----------------------------------------------------------------------------
#  Create the application and run it

app = OritatamiSimulatorApp()
app.MainLoop()

#---------------------------------------------------------------------------

